==> python-x2go_0.1.1.3.orig.tar.gz: python-x2go_0.1.1.3/x2go/forward.py <==
#!/usr/bin/env python

# Copyright (C) 2010-2011 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
#
# Python X2go is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Python X2go is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

"""\
Python Gevent based port forwarding server (openssh -L option) for the
proxying of graphical X2go elements.

"""
__NAME__ = "x2gofwtunnel-pylib"

# modules
import os, sys, copy

# gevent/greenlet
import gevent
from gevent import select, socket
from gevent.server import StreamServer

# Python X2go modules
import log
from defaults import X2GOCLIENT_OS as _X2GOCLIENT_OS
import x2go_exceptions

class X2goFwServer(StreamServer):
    """\
    L{X2goFwServer} implements a gevent's StreamServer based Paramiko/SSH port
    forwarding server.

    An L{X2goFwServer} class object is used to tunnel graphical trafic
    through an external proxy command launched by a C{X2goProxy*} backend.

    """
    def __init__ (self, listener, remote_host, remote_port, ssh_transport, session_instance=None, logger=None, loglevel=log.loglevel_DEFAULT,):
        """\
        @param listener: listen on TCP/IP socket C{(<IP>, <Port>)}
        @type listener: C{tuple}
        @param remote_host: hostname or IP of remote host (in case of X2go mostly 127.0.0.1)
        @type remote_host: C{str}
        @param remote_port: port of remote host
        @type remote_port: C{int}
        @param ssh_transport: a valid Paramiko/SSH transport object
        @type ssh_transport: C{instance}
        @param logger: you can pass an L{X2goLogger} object to the
            L{X2goFwServer} constructor
        @type logger: C{instance}
        @param loglevel: if no L{X2goLogger} object has been supplied a new one will be
            constructed with the given loglevel
        @type loglevel: C{int}

        """
        if logger is None:
            self.logger = log.X2goLogger(loglevel=loglevel)
        else:
            self.logger = copy.deepcopy(logger)
        self.logger.tag = __NAME__

        self.chan = None
        self.is_active = False
        self.keepalive = False
        self.chain_host = remote_host
        self.chain_port = remote_port
        self.ssh_transport = ssh_transport
        self.session_instance = session_instance

        self.fw_socket = None

        StreamServer.__init__(self, listener, self.x2go_forward_tunnel_handle)

    def x2go_forward_tunnel_handle(self, fw_socket, address):
        """\
        Handle for SSH/Paramiko forwarding tunnel.

        @param fw_socket: local end of the forwarding tunnel
        @type fw_socket: C{instance}
        @param address: unused/ignored
        @type address: C{tuple}

        """
        self.fw_socket = fw_socket

        _success = False
        _count = 0
        _maxwait = 20

        while not _success and _count < _maxwait:

            # it is recommended here to have passed on the session instance to this object...
            if self.session_instance:
                if not self.session_instance.is_connected():
                    break

            _count += 1
            try:
                self.chan = self.ssh_transport.open_channel('direct-tcpip',
                                                            (self.chain_host, self.chain_port),
                                                            self.fw_socket.getpeername())
                chan_peername = self.chan.getpeername()
                _success = True
            except Exception, e:
                self.logger('incoming request to %s:%d failed on attempt %d of %d: %s' % (self.chain_host,
                                                                                          self.chain_port,
                                                                                          _count,
                                                                                          _maxwait,
                                                                                          repr(e)),
                                                                                          loglevel=log.loglevel_WARN)
                gevent.sleep(.4)


        if not _success:
            self.logger('incoming request to %s:%d failed after %d attempts' % (self.chain_host,
                                                                                self.chain_port,
                                                                                _count),
                                                                                loglevel=log.loglevel_ERROR)
        else:
            # once we are here, we can presume the tunnel to be active...
            self.is_active = True

            if self.chan is None:
                self.logger('incoming request to [%s]:%d was rejected by the SSH server.' %
                            (self.chain_host, self.chain_port), loglevel=log.loglevel_ERROR)
                if self.session_instance:
                    self.session_instance.HOOK_forwarding_tunnel_setup_failed(chain_host=self.chain_host, chain_port=self.chain_port)
                return
            else:
                self.logger('connected!  Tunnel open %r -> %r -> %r' % (self.fw_socket.getpeername(),
                                                                chan_peername, (self.chain_host, self.chain_port)),
                                                                loglevel=log.loglevel_INFO)
            self.keepalive = True
            try:
                while self.keepalive:
                    r, w, x = select.select([self.fw_socket, self.chan], [], [])
                    if fw_socket in r:
                        data = fw_socket.recv(1024)
                        if len(data) == 0:
                            break
                        self.chan.send(data)
                    if self.chan in r:
                        data = self.chan.recv(1024)
                        if len(data) == 0:
                            break
                        fw_socket.send(data)
                self.close_channel()
                self.close_socket()
            except socket.error:
                pass

            self.is_active = False
            self.logger('Tunnel closed from %r' % (chan_peername,),
                        loglevel=log.loglevel_INFO)

    def close_channel(self):
        """\
        Close an open channel again.

        """
        #if self.chan is not None and _X2GOCLIENT_OS != "Windows":
        if self.chan is not None:
            try:
                if _X2GOCLIENT_OS != 'Windows':
                    self.chan.close()
                self.chan = None
            except EOFError:
                pass

    def close_socket(self):
        """\
        Close the forwarding tunnel's socket again.

        """
        _success = False
        _count = 0
        _maxwait = 20

        # try at least <_maxwait> times
        while not _success and _count < _maxwait:
            _count += 1
            try:
                self.close_channel()
                if self.fw_socket is not None:
                    self.fw_socket.close()
                _success = True
            except socket.error:
                gevent.sleep(.2)
                self.logger('could not close fw_tunnel socket, try again (%s of %s)' % (_count, _maxwait), loglevel=log.loglevel_WARN)

        if _count >= _maxwait:
            self.logger('forwarding tunnel to [%s]:%d could not be closed properly' % (self.chain_host, self.chain_port), loglevel=log.loglevel_WARN)

    def stop(self):
        """\
        Stop the forwarding tunnel.

        """
        self.close_socket()
        StreamServer.stop(self)


def start_forward_tunnel(local_host='127.0.0.1', local_port=22022,
                         remote_host='127.0.0.1', remote_port=22,
                         ssh_transport=None, 
                         session_instance=None,
                         logger=None, ):
    """\
    Setup up a Paramiko/SSH port forwarding tunnel (like openssh -L option).

    The tunnel is used to transport X2go graphics data through a proxy application like nxproxy.

    @param local_host: local starting point of the forwarding tunnel
    @type local_host: C{int}
    @param local_port: listen port of the local starting point
    @type local_port: C{int}
    @param remote_host: from the endpoint of the tunnel, connect to host C{<remote_host>}...
    @type remote_host: C{str}
    @param remote_port: ... on port C{<remote_port>}
    @type remote_port: C{int}
    @param ssh_transport: the Paramiko/SSH transport (i.e. the X2go sessions Paramiko/SSH transport object)
    @type ssh_transport: C{instance}
    @param session_instance: the L{X2goSession} instance that initiates this tunnel
    @type session_instance: C{instance}
    @param logger: an X2goLogger object
    @type logger: C{instance}

    @return: returns an L{X2goFwServer} instance
    @rtype: C{instance}

    """
    try:
        fw_server = X2goFwServer(listener=(local_host, local_port),
                                 remote_host=remote_host, remote_port=remote_port,
                                 ssh_transport=ssh_transport, session_instance=session_instance,
                                 logger=logger,
                                )
        try:
            fw_server.start()
            return fw_server
        except socket.error:
            pass
    except x2go_exceptions.X2goFwTunnelException:
        pass

    return None


def stop_forward_tunnel(fw_server):
    """\
    Tear down a given Paramiko/SSH port forwarding tunnel.

    @param fw_server: an L{X2goFwServer} instance as returned by the L{start_forward_tunnel()} function
    @type fw_server: C{instance}

    """
    if fw_server is not None:
        fw_server.keepalive = False
        gevent.sleep(.5)
        fw_server.stop()


if __name__ == '__main__':
    pass

==> python-x2go_0.1.1.8.orig.tar.gz: python-x2go_0.1.1.8/x2go/forward.py <==
#!/usr/bin/env python

# Copyright (C) 2010-2011 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
#
# Python X2go is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Python X2go is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

"""\
Python Gevent based port forwarding server (openssh -L option) for the
proxying of graphical X2go elements.

"""
__NAME__ = "x2gofwtunnel-pylib"

# modules
import os, sys, copy

# gevent/greenlet
import gevent
from gevent import select, socket
from gevent.server import StreamServer

# Python X2go modules
import log
from defaults import X2GOCLIENT_OS as _X2GOCLIENT_OS
import x2go_exceptions

class X2goFwServer(StreamServer):
    """\
    L{X2goFwServer} implements a gevent's StreamServer based Paramiko/SSH port
    forwarding server.

    An L{X2goFwServer} class object is used to tunnel graphical trafic
    through an external proxy command launched by a C{X2goProxy*} backend.

    """
    def __init__ (self, listener, remote_host, remote_port, ssh_transport, session_instance=None, logger=None, loglevel=log.loglevel_DEFAULT,):
        """\
        @param listener: listen on TCP/IP socket C{(<IP>, <Port>)}
        @type listener: C{tuple}
        @param remote_host: hostname or IP of remote host (in case of X2go mostly 127.0.0.1)
        @type remote_host: C{str}
        @param remote_port: port of remote host
        @type remote_port: C{int}
        @param ssh_transport: a valid Paramiko/SSH transport object
        @type ssh_transport: C{instance}
        @param logger: you can pass an L{X2goLogger} object to the
            L{X2goFwServer} constructor
        @type logger: C{instance}
        @param loglevel: if no L{X2goLogger} object has been supplied a new one will be
            constructed with the given loglevel
        @type loglevel: C{int}

        """
        if logger is None:
            self.logger = log.X2goLogger(loglevel=loglevel)
        else:
            self.logger = copy.deepcopy(logger)
        self.logger.tag = __NAME__

        self.chan = None
        self.is_active = False
        self.keepalive = False
        self.chain_host = remote_host
        self.chain_port = remote_port
        self.ssh_transport = ssh_transport
        self.session_instance = session_instance

        self.fw_socket = None

        StreamServer.__init__(self, listener, self.x2go_forward_tunnel_handle)

    def x2go_forward_tunnel_handle(self, fw_socket, address):
        """\
        Handle for SSH/Paramiko forwarding tunnel.

        @param fw_socket: local end of the forwarding tunnel
        @type fw_socket: C{instance}
        @param address: unused/ignored
        @type address: C{tuple}

        """
        self.fw_socket = fw_socket

        # disable Nagle algorithm in TCP/IP protocol
        self.fw_socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)

        _success = False
        _count = 0
        _maxwait = 20

        while not _success and _count < _maxwait:

            # it is recommended here to have passed on the session instance to this object...
            if self.session_instance:
                if not self.session_instance.is_connected():
                    break

            _count += 1
            try:
                self.chan = self.ssh_transport.open_channel('direct-tcpip',
                                                            (self.chain_host, self.chain_port),
                                                            self.fw_socket.getpeername())
                chan_peername = self.chan.getpeername()
                _success = True
            except Exception, e:
                self.logger('incoming request to %s:%d failed on attempt %d of %d: %s' % (self.chain_host,
                                                                                          self.chain_port,
                                                                                          _count,
                                                                                          _maxwait,
                                                                                          repr(e)),
                                                                                          loglevel=log.loglevel_WARN)
                gevent.sleep(.4)


        if not _success:
            self.logger('incoming request to %s:%d failed after %d attempts' % (self.chain_host,
                                                                                self.chain_port,
                                                                                _count),
                                                                                loglevel=log.loglevel_ERROR)
        else:
            # once we are here, we can presume the tunnel to be active...
            self.is_active = True

            if self.chan is None:
                self.logger('incoming request to [%s]:%d was rejected by the SSH server.' %
                            (self.chain_host, self.chain_port), loglevel=log.loglevel_ERROR)
                if self.session_instance:
                    self.session_instance.HOOK_forwarding_tunnel_setup_failed(chain_host=self.chain_host, chain_port=self.chain_port)
                return
            else:
                self.logger('connected!  Tunnel open %r -> %r -> %r' % (self.fw_socket.getpeername(),
                                                                chan_peername, (self.chain_host, self.chain_port)),
                                                                loglevel=log.loglevel_INFO)
            self.keepalive = True
            try:
                while self.keepalive:
                    r, w, x = select.select([self.fw_socket, self.chan], [], [])
                    if fw_socket in r:
                        data = fw_socket.recv(1024)
                        if len(data) == 0:
                            break
                        self.chan.send(data)
                    if self.chan in r:
                        data = self.chan.recv(1024)
                        if len(data) == 0:
                            break
                        fw_socket.send(data)
                self.close_channel()
                self.close_socket()
            except socket.error:
                pass

            self.is_active = False
            self.logger('Tunnel closed from %r' % (chan_peername,),
                        loglevel=log.loglevel_INFO)

    def close_channel(self):
        """\
        Close an open channel again.

        """
        #if self.chan is not None and _X2GOCLIENT_OS != "Windows":
        if self.chan is not None:
            try:
                if _X2GOCLIENT_OS != 'Windows':
                    self.chan.close()
                self.chan = None
            except EOFError:
                pass

    def close_socket(self):
        """\
        Close the forwarding tunnel's socket again.

        """
        _success = False
        _count = 0
        _maxwait = 20

        # try at least <_maxwait> times
        while not _success and _count < _maxwait:
            _count += 1
            try:
                self.close_channel()
                if self.fw_socket is not None:
                    self.fw_socket.close()
                _success = True
            except socket.error:
                gevent.sleep(.2)
                self.logger('could not close fw_tunnel socket, try again (%s of %s)' % (_count, _maxwait), loglevel=log.loglevel_WARN)

        if _count >= _maxwait:
            self.logger('forwarding tunnel to [%s]:%d could not be closed properly' % (self.chain_host, self.chain_port), loglevel=log.loglevel_WARN)

    def stop(self):
        """\
        Stop the forwarding tunnel.

        """
        self.close_socket()
        StreamServer.stop(self)


def start_forward_tunnel(local_host='127.0.0.1', local_port=22022,
                         remote_host='127.0.0.1', remote_port=22,
                         ssh_transport=None, 
                         session_instance=None,
                         logger=None, ):
    """\
    Setup up a Paramiko/SSH port forwarding tunnel (like openssh -L option).

    The tunnel is used to transport X2go graphics data through a proxy application like nxproxy.

    @param local_host: local starting point of the forwarding tunnel
    @type local_host: C{int}
    @param local_port: listen port of the local starting point
    @type local_port: C{int}
    @param remote_host: from the endpoint of the tunnel, connect to host C{<remote_host>}...
    @type remote_host: C{str}
    @param remote_port: ... on port C{<remote_port>}
    @type remote_port: C{int}
    @param ssh_transport: the Paramiko/SSH transport (i.e. the X2go sessions Paramiko/SSH transport object)
    @type ssh_transport: C{instance}
    @param session_instance: the L{X2goSession} instance that initiates this tunnel
    @type session_instance: C{instance}
    @param logger: an X2goLogger object
    @type logger: C{instance}

    @return: returns an L{X2goFwServer} instance
    @rtype: C{instance}

    """
    try:
        fw_server = X2goFwServer(listener=(local_host, local_port),
                                 remote_host=remote_host, remote_port=remote_port,
                                 ssh_transport=ssh_transport, session_instance=session_instance,
                                 logger=logger,
                                )
        try:
            fw_server.start()
            return fw_server
        except socket.error:
            pass
    except x2go_exceptions.X2goFwTunnelException:
        pass

    return None


def stop_forward_tunnel(fw_server):
    """\
    Tear down a given Paramiko/SSH port forwarding tunnel.

    @param fw_server: an L{X2goFwServer} instance as returned by the L{start_forward_tunnel()} function
    @type fw_server: C{instance}

    """
    if fw_server is not None:
        fw_server.keepalive = False
        gevent.sleep(.5)
        fw_server.stop()


if __name__ == '__main__':
    pass
